import copy

import collection
import const


class Sampler(object):

    def __init__(self, sound_file, index_list, direction_list):

        #: Fragment. Analogous to a real-world sound file, with event objects
        #: representing samples.
        self.sound_file = sound_file

        #: List. A list of indices that Sampler's pointer will pause on as it
        #: indexes *sound_file.event_list*.
        self.index_list = index_list

        #: List. A list of directions that Sampler's pointer will index in. The
        #: length of this list must be one less than *index_list*.
        self.direction_list = direction_list

        #: Integer. A pointer to *sound_file.event_list*.
        self.pointer = None

    def __repr__(self):
        return "%s %s %s %s" % (self.sound_file,
                                self.index_list,
                                self.direction_list,
                                self.pointer)

    # sample
    # ------------------------------------------------------------------- #

    def sample(self):
        """
        Traverse *sound_file* according to *index_list* and *direction_list*,
        copying each event to output.

        :returns: Section

        """
        s = collection.Section([], 0.0)
        for i in range(len(self.direction_list)):
            f = collection.Fragment([], 0.0)
            direction = self.direction_list[i]
            self.pointer = self.index_list[i] % \
                len(self.sound_file.event_list)
            next_index = self.index_list[i + 1] % \
                len(self.sound_file.event_list)
            while True:
                self._write(f)
                self._move_pointer(direction)
                if self.pointer is next_index:
                    self._write(f)
                    break
            s.fragment_list.append(f)
        s.queue()
        return s

    def _move_pointer(self, direction):
        """
        Move *pointer* one step in direction *direction*, wrapping as necessary.

        :param direction: direction to move, either backward or forward
        :type direction: integer

        """
        if direction is const.FORWARD:
            self.pointer += 1
            if self.pointer > len(self.sound_file.event_list) - 1:
                self.pointer = 0
        if direction is const.BACKWARD:
            self.pointer -= 1
            if self.pointer < 0:
                self.pointer += len(self.sound_file.event_list)

    def _write(self, fragment):
        """
        Write the event that *pointer* is currently referencing to *fragment*.

        :param fragment: fragment to write to
        :type fragment: Fragment

        """
        event = self.sound_file.event_list[self.pointer]
        event = copy.deepcopy(event)
        fragment.event_list.append(event)
